import os
import time
import signal
from subprocess import call, check_call, Popen

import jujuresources
from path import Path
from charmhelpers.core import host
from charmhelpers.core import hookenv
from charmhelpers.core import charmdata
from charmhelpers.contrib.bigdata import utils


class Hive(object):
    hive_ports = [10000]
    hive_user = 'hive'

    HIVE_VERSION = '1.0.0'
    DIRS = {
        'hive': Path('/usr/lib/hive'),
        'hive_conf': Path('/etc/hive/conf.juju'),
        'hive_installed': Path('/usr/lib/hive/apache-hive-%s-bin' % HIVE_VERSION),
        'hive_real_conf': Path('/etc/hive/conf'),
    }

    def __init__(self, hadoop):
        self.hadoop = hadoop

    def is_installed(self):
        return charmdata.kv.get('hive.installed')

    def install(self, force=False):
        if not force and self.is_installed():
            return
        host.mkdir(self.DIRS['hive'], perms=0o777)
        host.adduser(self.hive_user, group='hadoop')
        mirror_url = hookenv.config()['resources_mirror']
        jujuresources.fetch('hive-%s' % self.hadoop_base.cpu_arch, mirror_url=mirror_url)
        jujuresources.install('hive-%s' % self.hadoop_base.cpu_arch, destination=self.DIRS['hive'])
        self.setup_hive_config()
        self.configure_hive()
        charmdata.kv.set('hive.installed', True)

    def setup_hive_config(self):
        # copy default config into alternate dir
        conf_dir = self.DIRS['hive_installed'] / 'conf'
        self.DIRS['hive_conf'].rmtree_p()
        conf_dir.copytree(self.DIRS['hive_conf'])
        hive_env = self.DIRS['hive_conf'] / 'hive-env.sh'
        if not hive_env.exists():
            (self.DIRS['hive_conf'] / 'hive-env.sh.template').copy(hive_env)
        hive_site = self.DIRS['hive_conf'] / 'hive-site.xml'
        if not hive_site.exists():
            (self.DIRS['hive_conf'] / 'hive-default.xml.template').copy(hive_site)
        # setup to use alternates
        if call(['update-alternatives', '--list', 'hive-conf']) != 0:
            check_call(['update-alternatives', '--install',
                        self.DIRS['hive_real_conf'], 'hive-conf',
                        self.DIRS['hive_conf'], '50'])

    def configure_hive(self):
        config = hookenv.config()
        hive_site = self.DIRS['hive_conf'] / 'hive-site.xml'
        with utils.xmlpropmap_edit_in_place(hive_site) as props:
            mysql = charmdata.kv.get('relations.filtered.db')[0]
            props['javax.jdo.option.ConnectionURL'] = "jdbc:mysql://{}/{}".format(
                mysql['host'], mysql['db']
            )
            props['javax.jdo.option.ConnectionUserName'] = mysql['user']
            props['javax.jdo.option.ConnectionPassword'] = mysql['password']
            props['javax.jdo.option.ConnectionDriverName'] = "com.mysql.jdbc.Driver"
            props['hive.hwi.war.file'] = self.DIRS['hive_installed'] / "lib/hive-hwi-%s.jar" % self.HIVE_VERSION
        self.run('hive', 'bin/schematool', '-initSchema', '-dbType', 'mysql')
        self.hadoop_base.run('hdfs', 'bin/hdfs', 'dfs', '-mkdir', '-p', '/user/hive/warehouse')
        self.hadoop_base.run('hdfs', 'bin/hdfs', 'dfs', '-chown', '-R', 'hive', '/user/hive/warehouse')

        hive_env = self.DIRS['hive_conf'] / 'hive-env.sh'
        utils.re_edit_in_place(hive_env, {
            r'.*export HADOOP_HEAPSIZE *=.*': 'export HADOOP_HEAPSIZE=%s' % config['heap'],
        })

    def run(self, user, command, *args):
        """
        Run a Hive command as the `hive` user in the foreground, checking the return code.

        :param str command: Command to run, prefixed with `bin/` or `sbin/`
        :param list args: Additional args to pass to the command
        """
        parts = [self.DIRS['hive_installed'] / command] + list(args)
        quoted = ' '.join("'%s'" % p for p in parts)
        check_call(['su', user, '-c', quoted])

    def run_bg(self, user, command, *args):
        """
        Run a Hive command as the `hive` user in the background.

        :param str command: Command to run, prefixed with `bin/` or `sbin/`
        :param list args: Additional args to pass to the command
        """
        parts = [self.DIRS['hive_installed'] / command] + list(args)
        quoted = ' '.join("'%s'" % p for p in parts)
        Popen(['su', user, '-c', quoted])

    def start_hive(self):
        if not utils.jps('HiveServer2'):
            self.hadoop.run_bg(
                'hive', 'bin/hive',
                '--config', self.DIRS['hive_conf'],
                '--service', 'hiveserver2')
            time.sleep(5)

    def stop_hive(self):
        hive_pids = utils.jps('HiveServer2')
        for pid in hive_pids:
            os.kill(int(pid), signal.SIGTERM)
